@@ -15,15 +15,15 @@
 """Graphical support for Bazaar using GTK.
 
 This plugin includes:
-commit-notify     Start the graphical notifier of commits.
 gannotate         GTK+ annotate. 
 gbranch           GTK+ branching. 
 gcheckout         GTK+ checkout. 
 gcommit           GTK+ commit dialog.
 gconflicts        GTK+ conflicts. 
 gdiff             Show differences in working tree in a GTK+ Window. 
-ghandle-patch     Display and optionally merge a merge directive or patch.
 ginit             Initialise a new branch.
+gloom             GTK+ loom browse dialog
+gmerge            GTK+ merge dialog
 gmissing          GTK+ missing revisions dialog. 
 gpreferences      GTK+ preferences dialog. 
 gpush             GTK+ push.
@@ -37,7 +37,7 @@ import sys
 
 import bzrlib
 
-version_info = (0, 94, 0, 'final', 1)
+version_info = (0, 95, 0, 'final', 1)
 
 if version_info[3] == 'final':
     version_string = '%d.%d.%d' % version_info[:3]
@@ -45,7 +45,7 @@ else:
     version_string = '%d.%d.%d%s%d' % version_info
 __version__ = version_string
 
-required_bzrlib = (1, 3)
+required_bzrlib = (1, 6)
 
 def check_bzrlib_version(desired):
     """Check that bzrlib is compatible.
@@ -105,36 +105,40 @@ def set_ui_factory():
     bzrlib.ui.ui_factory = GtkUIFactory()
 
 
-def data_path():
-    return os.path.dirname(__file__)
-
-
-def icon_path(*args):
-    basedirs = [os.path.join(data_path()),
+def data_basedirs():
+    return [os.path.dirname(__file__),
              "/usr/share/bzr-gtk", 
              "/usr/local/share/bzr-gtk"]
-    for basedir in basedirs:
-        path = os.path.join(basedir, 'icons', *args)
+
+
+def data_path(*args):
+    for basedir in data_basedirs():
+        path = os.path.join(basedir, *args)
         if os.path.exists(path):
             return path
     return None
 
 
+def icon_path(*args):
+    return data_path(os.path.join('icons', *args))
+
+
+def open_display():
+    pygtk = import_pygtk()
+    try:
+        import gtk
+    except RuntimeError, e:
+        if str(e) == "could not open display":
+            raise NoDisplayError
+    set_ui_factory()
+    return gtk
+ 
+
 class GTKCommand(Command):
     """Abstract class providing GTK specific run commands."""
 
-    def open_display(self):
-        pygtk = import_pygtk()
-        try:
-            import gtk
-        except RuntimeError, e:
-            if str(e) == "could not open display":
-                raise NoDisplayError
-        set_ui_factory()
-        return gtk
-
     def run(self):
-        self.open_display()
+        open_display()
         dialog = self.get_gtk_dialog(os.path.abspath('.'))
         dialog.run()
 
@@ -168,12 +172,30 @@ class cmd_gpush(GTKCommand):
 
     def run(self, location="."):
         (br, path) = branch.Branch.open_containing(location)
-        self.open_display()
-        from push import PushDialog
+        open_display()
+        from bzrlib.plugins.gtk.push import PushDialog
         dialog = PushDialog(br.repository, br.last_revision(), br)
         dialog.run()
 
 
+class cmd_gloom(GTKCommand):
+    """ GTK+ loom.
+    
+    """
+    takes_args = [ "location?" ]
+
+    def run(self, location="."):
+        try:
+            (tree, path) = workingtree.WorkingTree.open_containing(location)
+            br = tree.branch
+        except NoWorkingTree, e:
+            (br, path) = branch.Branch.open_containing(location)
+            tree = None
+        open_display()
+        from bzrlib.plugins.gtk.loom import LoomDialog
+        dialog = LoomDialog(br, tree)
+        dialog.run()
+
 
 class cmd_gdiff(GTKCommand):
     """Show differences in working tree in a GTK+ Window.
@@ -193,12 +215,12 @@ class cmd_gdiff(GTKCommand):
             if revision is not None:
                 if len(revision) == 1:
                     tree1 = wt
-                    revision_id = revision[0].in_history(branch).rev_id
+                    revision_id = revision[0].as_revision_id(tree1.branch)
                     tree2 = branch.repository.revision_tree(revision_id)
                 elif len(revision) == 2:
-                    revision_id_0 = revision[0].in_history(branch).rev_id
+                    revision_id_0 = revision[0].as_revision_id(branch)
                     tree2 = branch.repository.revision_tree(revision_id_0)
-                    revision_id_1 = revision[1].in_history(branch).rev_id
+                    revision_id_1 = revision[1].as_revision_id(branch)
                     tree1 = branch.repository.revision_tree(revision_id_1)
             else:
                 tree1 = wt
@@ -231,7 +253,7 @@ def start_viz_window(branch, revisions, limit=None):
     
     :return: The viz window object.
     """
-    from viz import BranchWindow
+    from bzrlib.plugins.gtk.viz import BranchWindow
     return BranchWindow(branch, revisions, limit)
 
 
@@ -261,8 +283,7 @@ class cmd_visualise(Command):
             if revision is None:
                 revids.append(br.last_revision())
             else:
-                (revno, revid) = revision[0].in_history(br)
-                revids.append(revid)
+                revids.append(revision[0].as_revision_id(br))
         import gtk
         pp = start_viz_window(br, revids, limit)
         pp.connect("destroy", lambda w: gtk.main_quit())
@@ -287,7 +308,7 @@ class cmd_gannotate(GTKCommand):
     aliases = ["gblame", "gpraise"]
     
     def run(self, filename, all=False, plain=False, line='1', revision=None):
-        gtk = self.open_display()
+        gtk = open_display()
 
         try:
             line = int(line)
@@ -312,7 +333,7 @@ class cmd_gannotate(GTKCommand):
         if revision is not None:
             if len(revision) != 1:
                 raise BzrCommandError("Only 1 revion may be specified.")
-            revision_id = revision[0].in_history(br).rev_id
+            revision_id = revision[0].as_revision_id(br)
             tree = br.repository.revision_tree(revision_id)
         else:
             revision_id = getattr(tree, 'get_revision_id', lambda: None)()
@@ -346,7 +367,7 @@ class cmd_gcommit(GTKCommand):
 
     def run(self, filename=None):
         import os
-        self.open_display()
+        open_display()
         from commit import CommitDialog
         from bzrlib.errors import (BzrCommandError,
                                    NotBranchError,
@@ -359,8 +380,8 @@ class cmd_gcommit(GTKCommand):
             br = wt.branch
         except NoWorkingTree, e:
             from dialog import error_dialog
-            error_dialog(_('Directory does not have a working tree'),
-                         _('Operation aborted.'))
+            error_dialog(_i18n('Directory does not have a working tree'),
+                         _i18n('Operation aborted.'))
             return 1 # should this be retval=3?
 
         # It is a good habit to keep things locked for the duration, but it
@@ -387,13 +408,13 @@ class cmd_gstatus(GTKCommand):
 
     def run(self, path='.', revision=None):
         import os
-        gtk = self.open_display()
+        gtk = open_display()
         from status import StatusDialog
         (wt, wt_path) = workingtree.WorkingTree.open_containing(path)
         
         if revision is not None:
             try:
-                revision_id = revision[0].in_history(wt.branch).rev_id
+                revision_id = revision[0].as_revision_id(wt.branch)
             except:
                 from bzrlib.errors import BzrError
                 raise BzrError('Revision %r doesn\'t exist' % revision[0].user_spec )
@@ -411,7 +432,7 @@ class cmd_gsend(GTKCommand):
     """
     def run(self):
         (br, path) = branch.Branch.open_containing(".")
-        gtk = self.open_display()
+        gtk = open_display()
         from bzrlib.plugins.gtk.mergedirective import SendMergeDirectiveDialog
         from StringIO import StringIO
         dialog = SendMergeDirectiveDialog(br)
@@ -433,7 +454,7 @@ class cmd_gconflicts(GTKCommand):
     """
     def run(self):
         (wt, path) = workingtree.WorkingTree.open_containing('.')
-        self.open_display()
+        open_display()
         from bzrlib.plugins.gtk.conflicts import ConflictsDialog
         dialog = ConflictsDialog(wt)
         dialog.run()
@@ -444,12 +465,35 @@ class cmd_gpreferences(GTKCommand):
 
     """
     def run(self):
-        self.open_display()
+        open_display()
         from bzrlib.plugins.gtk.preferences import PreferencesWindow
         dialog = PreferencesWindow()
         dialog.run()
 
 
+class cmd_gmerge(Command):
+    """ GTK+ merge dialog
+    
+    """
+    takes_args = ["merge_from_path?"]
+    def run(self, merge_from_path=None):
+        from bzrlib import workingtree
+        from bzrlib.plugins.gtk.dialog import error_dialog
+        from bzrlib.plugins.gtk.merge import MergeDialog
+        
+        (wt, path) = workingtree.WorkingTree.open_containing('.')
+        old_tree = wt.branch.repository.revision_tree(wt.branch.last_revision())
+        delta = wt.changes_from(old_tree)
+        if len(delta.added) or len(delta.removed) or len(delta.renamed) or len(delta.modified):
+            error_dialog(_i18n('There are local changes in the branch'),
+                         _i18n('Please commit or revert the changes before merging.'))
+        else:
+            parent_branch_path = wt.branch.get_parent()
+            merge = MergeDialog(wt, path, parent_branch_path)
+            response = merge.run()
+            merge.destroy()
+
+
 class cmd_gmissing(Command):
     """ GTK+ missing revisions dialog.
 
@@ -488,7 +532,7 @@ class cmd_gmissing(Command):
 
 class cmd_ginit(GTKCommand):
     def run(self):
-        self.open_display()
+        open_display()
         from initialize import InitDialog
         dialog = InitDialog(os.path.abspath(os.path.curdir))
         dialog.run()
@@ -498,7 +542,7 @@ class cmd_gtags(GTKCommand):
     def run(self):
         br = branch.Branch.open_containing('.')[0]
         
-        gtk = self.open_display()
+        gtk = open_display()
         from tags import TagsWindow
         window = TagsWindow(br)
         window.show()
@@ -513,6 +557,7 @@ commands = [
     cmd_gconflicts, 
     cmd_gdiff,
     cmd_ginit,
+    cmd_gmerge,
     cmd_gmissing, 
     cmd_gpreferences, 
     cmd_gpush, 
@@ -522,81 +567,17 @@ commands = [
     cmd_visualise
     ]
 
+try:
+    from bzrlib.plugins import loom
+except ImportError:
+    pass # Loom plugin doesn't appear to be present
+else:
+    commands.append(cmd_gloom)
+
 for cmd in commands:
     register_command(cmd)
 
 
-class cmd_commit_notify(GTKCommand):
-    """Run the bzr commit notifier.
-
-    This is a background program which will pop up a notification on the users
-    screen when a commit occurs.
-    """
-
-    def run(self):
-        from notify import NotifyPopupMenu
-        gtk = self.open_display()
-        menu = NotifyPopupMenu()
-        icon = gtk.status_icon_new_from_file(icon_path("bzr-icon-64.png"))
-        icon.connect('popup-menu', menu.display)
-
-        import cgi
-        import dbus
-        import dbus.service
-        import pynotify
-        from bzrlib.bzrdir import BzrDir
-        from bzrlib import errors
-        from bzrlib.osutils import format_date
-        from bzrlib.transport import get_transport
-        if getattr(dbus, 'version', (0,0,0)) >= (0,41,0):
-            import dbus.glib
-        BROADCAST_INTERFACE = "org.bazaarvcs.plugins.dbus.Broadcast"
-        bus = dbus.SessionBus()
-
-        def catch_branch(revision_id, urls):
-            # TODO: show all the urls, or perhaps choose the 'best'.
-            url = urls[0]
-            try:
-                if isinstance(revision_id, unicode):
-                    revision_id = revision_id.encode('utf8')
-                transport = get_transport(url)
-                a_dir = BzrDir.open_from_transport(transport)
-                branch = a_dir.open_branch()
-                revno = branch.revision_id_to_revno(revision_id)
-                revision = branch.repository.get_revision(revision_id)
-                summary = 'New revision %d in %s' % (revno, url)
-                body  = 'Committer: %s\n' % revision.committer
-                body += 'Date: %s\n' % format_date(revision.timestamp,
-                    revision.timezone)
-                body += '\n'
-                body += revision.message
-                body = cgi.escape(body)
-                nw = pynotify.Notification(summary, body)
-                def start_viz(notification=None, action=None, data=None):
-                    """Start the viz program."""
-                    pp = start_viz_window(branch, revision_id)
-                    pp.show()
-                def start_branch(notification=None, action=None, data=None):
-                    """Start a Branch dialog"""
-                    from bzrlib.plugins.gtk.branch import BranchDialog
-                    bd = BranchDialog(remote_path=url)
-                    bd.run()
-                nw.add_action("inspect", "Inspect", start_viz, None)
-                nw.add_action("branch", "Branch", start_branch, None)
-                nw.set_timeout(5000)
-                nw.show()
-            except Exception, e:
-                print e
-                raise
-        bus.add_signal_receiver(catch_branch,
-                                dbus_interface=BROADCAST_INTERFACE,
-                                signal_name="Revision")
-        pynotify.init("bzr commit-notify")
-        gtk.main()
-
-register_command(cmd_commit_notify)
-
-
 class cmd_gselftest(GTKCommand):
     """Version of selftest that displays a notification at the end"""
 
@@ -693,46 +674,13 @@ class cmd_test_gtk(GTKCommand):
 register_command(cmd_test_gtk)
 
 
-class cmd_ghandle_patch(GTKCommand):
-    """Display a patch or merge directive, possibly merging.
-
-    This is a helper, meant to be launched from other programs like browsers
-    or email clients.  Since these programs often do not allow parameters to
-    be provided, a "handle-patch" script is included.
-    """
-
-    takes_args = ['path']
-
-    def run(self, path):
-        try:
-            from bzrlib.plugins.gtk.diff import (DiffWindow,
-                                                 MergeDirectiveWindow)
-            lines = open(path, 'rb').readlines()
-            lines = [l.replace('\r\n', '\n') for l in lines]
-            try:
-                directive = merge_directive.MergeDirective.from_lines(lines)
-            except errors.NotAMergeDirective:
-                window = DiffWindow()
-                window.set_diff_text(path, lines)
-            else:
-                window = MergeDirectiveWindow(directive, path)
-                window.set_diff_text(path, directive.patch.splitlines(True))
-            window.show()
-            gtk = self.open_display()
-            window.connect("destroy", gtk.main_quit)
-        except Exception, e:
-            from dialog import error_dialog
-            error_dialog('Error', str(e))
-            raise
-        gtk.main()
-
-
-register_command(cmd_ghandle_patch)
-
 
 import gettext
 gettext.install('olive-gtk')
 
+# Let's create a specialized alias to protect '_' from being erased by other
+# uses of '_' as an anonymous variable (think pdb for one).
+_i18n = gettext.gettext
 
 class NoDisplayError(BzrCommandError):
     """gtk could not find a proper display"""
