@@ -41,6 +41,7 @@ from bzrlib.diff import show_diff_trees, internal_diff
 from bzrlib.errors import NoSuchFile
 from bzrlib.patches import parse_patches
 from bzrlib.trace import warning
+from bzrlib.plugins.gtk import _i18n
 from bzrlib.plugins.gtk.window import Window
 from dialog import error_dialog, info_dialog, warning_dialog
 
@@ -72,15 +73,15 @@ class DiffFileView(gtk.ScrolledWindow):
             self.buffer.set_language(gsl)
             self.buffer.set_highlight(True)
 
-            sourceview = gtksourceview.SourceView(self.buffer)
+            self.sourceview = gtksourceview.SourceView(self.buffer)
         else:
             self.buffer = gtk.TextBuffer()
-            sourceview = gtk.TextView(self.buffer)
+            self.sourceview = gtk.TextView(self.buffer)
 
-        sourceview.set_editable(False)
-        sourceview.modify_font(pango.FontDescription("Monospace"))
-        self.add(sourceview)
-        sourceview.show()
+        self.sourceview.set_editable(False)
+        self.sourceview.modify_font(pango.FontDescription("Monospace"))
+        self.add(self.sourceview)
+        self.sourceview.show()
 
     @staticmethod
     def apply_gedit_colors(lang):
@@ -304,7 +305,7 @@ class DiffWidget(gtk.HPaned):
         scrollwin.set_shadow_type(gtk.SHADOW_IN)
         self.pack1(scrollwin)
         scrollwin.show()
-
+        
         self.model = gtk.TreeStore(str, str)
         self.treeview = gtk.TreeView(self.model)
         self.treeview.set_headers_visible(False)
@@ -327,16 +328,17 @@ class DiffWidget(gtk.HPaned):
         """
         # The diffs of the  selected file: a scrollable source or
         # text view
-        self.diff_view = DiffFileView()
+
+    def set_diff_text_sections(self, sections):
+        if getattr(self, 'diff_view', None) is None:
+            self.diff_view = DiffFileView()
+            self.pack2(self.diff_view)
         self.diff_view.show()
-        self.pack2(self.diff_view)
-        self.model.append(None, [ "Complete Diff", "" ])
-        self.diff_view._diffs[None] = ''.join(lines)
-        for patch in parse_patches(lines):
-            oldname = patch.oldname.split('\t')[0]
-            newname = patch.newname.split('\t')[0]
-            self.model.append(None, [oldname, newname])
+        for oldname, newname, patch in sections:
             self.diff_view._diffs[newname] = str(patch)
+            if newname is None:
+                newname = ''
+            self.model.append(None, [oldname, newname])
         self.diff_view.show_diff(None)
 
     def set_diff(self, rev_tree, parent_tree):
@@ -345,8 +347,9 @@ class DiffWidget(gtk.HPaned):
         Compares the two trees and populates the window with the
         differences.
         """
-        self.diff_view = DiffView()
-        self.pack2(self.diff_view)
+        if getattr(self, 'diff_view', None) is None:
+            self.diff_view = DiffView()
+            self.pack2(self.diff_view)
         self.diff_view.show()
         self.diff_view.set_trees(rev_tree, parent_tree)
         self.rev_tree = rev_tree
@@ -379,6 +382,7 @@ class DiffWidget(gtk.HPaned):
                 self.model.append(titer, [ path, path ])
 
         self.treeview.expand_all()
+        self.diff_view.show_diff(None)
 
     def set_file(self, file_path):
         """Select the current file to display"""
@@ -401,9 +405,15 @@ class DiffWidget(gtk.HPaned):
             return
         elif specific_files == [ "" ]:
             specific_files = None
-
+        
         self.diff_view.show_diff(specific_files)
-
+    
+    def _on_wraplines_toggled(self, widget=None, wrap=False):
+        """Callback for when the wrap lines checkbutton is toggled"""
+        if wrap or widget.get_active():
+            self.diff_view.sourceview.set_wrap_mode(gtk.WRAP_WORD)
+        else:
+            self.diff_view.sourceview.set_wrap_mode(gtk.WRAP_NONE)
 
 class DiffWindow(Window):
     """Diff window.
@@ -412,7 +422,7 @@ class DiffWindow(Window):
     differences between two revisions on a branch.
     """
 
-    def __init__(self, parent=None):
+    def __init__(self, parent=None, operations=None):
         Window.__init__(self, parent)
         self.set_border_width(0)
         self.set_title("bzrk diff")
@@ -423,36 +433,99 @@ class DiffWindow(Window):
         width = int(monitor.width * 0.66)
         height = int(monitor.height * 0.66)
         self.set_default_size(width, height)
+        self.construct(operations)
 
-        self.construct()
-
-    def construct(self):
+    def construct(self, operations):
         """Construct the window contents."""
         self.vbox = gtk.VBox()
         self.add(self.vbox)
         self.vbox.show()
-        hbox = self._get_button_bar()
-        if hbox is not None:
-            self.vbox.pack_start(hbox, expand=False, fill=True)
         self.diff = DiffWidget()
-        self.vbox.add(self.diff)
+        self.vbox.pack_end(self.diff, True, True, 0)
         self.diff.show_all()
-
-    def _get_button_bar(self):
+        # Build after DiffWidget to connect signals
+        menubar = self._get_menu_bar()
+        self.vbox.pack_start(menubar, False, False, 0)
+        hbox = self._get_button_bar(operations)
+        if hbox is not None:
+            self.vbox.pack_start(hbox, False, True, 0)
+        
+    
+    def _get_menu_bar(self):
+        menubar = gtk.MenuBar()
+        # View menu
+        mb_view = gtk.MenuItem(_i18n("_View"))
+        mb_view_menu = gtk.Menu()
+        mb_view_wrapsource = gtk.CheckMenuItem(_i18n("Wrap _Long Lines"))
+        mb_view_wrapsource.connect('activate', self.diff._on_wraplines_toggled)
+        mb_view_wrapsource.show()
+        mb_view_menu.append(mb_view_wrapsource)
+        mb_view.show()
+        mb_view.set_submenu(mb_view_menu)
+        mb_view.show()
+        menubar.append(mb_view)
+        menubar.show()
+        return menubar
+    
+    def _get_button_bar(self, operations):
         """Return a button bar to use.
 
         :return: None, meaning that no button bar will be used.
         """
-        return None
+        if operations is None:
+            return None
+        hbox = gtk.HButtonBox()
+        hbox.set_layout(gtk.BUTTONBOX_START)
+        for title, method in operations:
+            merge_button = gtk.Button(title)
+            merge_button.show()
+            merge_button.set_relief(gtk.RELIEF_NONE)
+            merge_button.connect("clicked", method)
+            hbox.pack_start(merge_button, expand=False, fill=True)
+        hbox.show()
+        return hbox
 
-    def set_diff_text(self, description, lines):
-        """Set the diff from a text.
+    def _get_merge_target(self):
+        d = gtk.FileChooserDialog('Merge branch', self,
+                                  gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
+                                  buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
+                                           gtk.STOCK_CANCEL,
+                                           gtk.RESPONSE_CANCEL,))
+        try:
+            result = d.run()
+            if result != gtk.RESPONSE_OK:
+                raise SelectCancelled()
+            return d.get_current_folder_uri()
+        finally:
+            d.destroy()
 
-        The diff must be in unified diff format, and will be parsed to
-        determine filenames.
-        """
-        self.diff.set_diff_text(lines)
-        self.set_title(description + " - bzrk diff")
+    def _merge_successful(self):
+        # No conflicts found.
+        info_dialog(_i18n('Merge successful'),
+                    _i18n('All changes applied successfully.'))
+
+    def _conflicts(self):
+        warning_dialog(_i18n('Conflicts encountered'),
+                       _i18n('Please resolve the conflicts manually'
+                             ' before committing.'))
+
+    def _handle_error(self, e):
+        error_dialog('Error', str(e))
+
+    def _get_save_path(self, basename):
+        d = gtk.FileChooserDialog('Save As', self,
+                                  gtk.FILE_CHOOSER_ACTION_SAVE,
+                                  buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
+                                           gtk.STOCK_CANCEL,
+                                           gtk.RESPONSE_CANCEL,))
+        d.set_current_name(basename)
+        try:
+            result = d.run()
+            if result != gtk.RESPONSE_OK:
+                raise SelectCancelled()
+            return urlutils.local_path_from_url(d.get_uri())
+        finally:
+            d.destroy()
 
     def set_diff(self, description, rev_tree, parent_tree):
         """Set the differences showed by this window.
@@ -467,38 +540,64 @@ class DiffWindow(Window):
         self.diff.set_file(file_path)
 
 
-class MergeDirectiveWindow(DiffWindow):
+class DiffController(object):
 
-    def __init__(self, directive, path):
-        DiffWindow.__init__(self, None)
-        self._merge_target = None
-        self.directive = directive
+    def __init__(self, path, patch, window=None):
         self.path = path
+        self.patch = patch
+        if window is None:
+            window = DiffWindow(operations=self._provide_operations())
+            self.initialize_window(window)
+        self.window = window
 
-    def _get_button_bar(self):
-        """The button bar has only the Merge button"""
-        merge_button = gtk.Button('Merge')
-        merge_button.show()
-        merge_button.set_relief(gtk.RELIEF_NONE)
-        merge_button.connect("clicked", self.perform_merge)
+    def initialize_window(self, window):
+        window.diff.set_diff_text_sections(self.get_diff_sections())
+        window.set_title(self.path + " - diff")
 
-        save_button = gtk.Button('Save')
-        save_button.show()
-        save_button.set_relief(gtk.RELIEF_NONE)
-        save_button.connect("clicked", self.perform_save)
+    def get_diff_sections(self):
+        yield "Complete Diff", None, ''.join(self.patch)
+        for patch in parse_patches(self.patch):
+            oldname = patch.oldname.split('\t')[0]
+            newname = patch.newname.split('\t')[0]
+            yield oldname, newname, str(patch)
 
-        hbox = gtk.HButtonBox()
-        hbox.set_layout(gtk.BUTTONBOX_START)
-        hbox.pack_start(merge_button, expand=False, fill=True)
-        hbox.pack_start(save_button, expand=False, fill=True)
-        hbox.show()
-        return hbox
-
-    def perform_merge(self, window):
+    def perform_save(self, window):
         try:
-            tree = self._get_merge_target()
+            save_path = self.window._get_save_path(osutils.basename(self.path))
         except SelectCancelled:
             return
+        source = open(self.path, 'rb')
+        try:
+            target = open(save_path, 'wb')
+            try:
+                osutils.pumpfile(source, target)
+            finally:
+                target.close()
+        finally:
+            source.close()
+
+    def _provide_operations(self):
+        return [('Save', self.perform_save)]
+
+
+class MergeDirectiveController(DiffController):
+
+    def __init__(self, path, directive, window=None):
+        DiffController.__init__(self, path, directive.patch.splitlines(True),
+                                window)
+        self.directive = directive
+        self.merge_target = None
+
+    def _provide_operations(self):
+        return [('Merge', self.perform_merge), ('Save', self.perform_save)]
+
+    def perform_merge(self, window):
+        if self.merge_target is None:
+            try:
+                self.merge_target = self.window._get_merge_target()
+            except SelectCancelled:
+                return
+        tree = workingtree.WorkingTree.open(self.merge_target)
         tree.lock_write()
         try:
             try:
@@ -509,64 +608,16 @@ class MergeDirectiveWindow(DiffWindow):
                 conflict_count = merger.do_merge()
                 merger.set_pending()
                 if conflict_count == 0:
-                    # No conflicts found.
-                    info_dialog(_('Merge successful'),
-                                _('All changes applied successfully.'))
+                    self.window._merge_successful()
                 else:
+                    self.window._conflicts()
                     # There are conflicts to be resolved.
-                    warning_dialog(_('Conflicts encountered'),
-                                   _('Please resolve the conflicts manually'
-                                     ' before committing.'))
-                self.destroy()
+                self.window.destroy()
             except Exception, e:
-                error_dialog('Error', str(e))
+                self.window._handle_error(e)
         finally:
             tree.unlock()
 
-    def _get_merge_target(self):
-        if self._merge_target is not None:
-            return self._merge_target
-        d = gtk.FileChooserDialog('Merge branch', self,
-                                  gtk.FILE_CHOOSER_ACTION_SELECT_FOLDER,
-                                  buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
-                                           gtk.STOCK_CANCEL,
-                                           gtk.RESPONSE_CANCEL,))
-        try:
-            result = d.run()
-            if result != gtk.RESPONSE_OK:
-                raise SelectCancelled()
-            uri = d.get_current_folder_uri()
-        finally:
-            d.destroy()
-        return workingtree.WorkingTree.open(uri)
-
-    def perform_save(self, window):
-        d = gtk.FileChooserDialog('Save As', self,
-                                  gtk.FILE_CHOOSER_ACTION_SAVE,
-                                  buttons=(gtk.STOCK_OK, gtk.RESPONSE_OK,
-                                           gtk.STOCK_CANCEL,
-                                           gtk.RESPONSE_CANCEL,))
-        d.set_current_name(osutils.basename(self.path))
-        try:
-            try:
-                result = d.run()
-                if result != gtk.RESPONSE_OK:
-                    raise SelectCancelled()
-                uri = d.get_uri()
-            finally:
-                d.destroy()
-        except SelectCancelled:
-            return
-        source = open(self.path, 'rb')
-        try:
-            target = open(urlutils.local_path_from_url(uri), 'wb')
-            try:
-                target.write(source.read())
-            finally:
-                target.close()
-        finally:
-            source.close()
-
 
 def iter_changes_to_status(source, target):
     """Determine the differences between trees.
