@@ -20,11 +20,13 @@ pygtk.require("2.0")
 import gtk
 import pango
 import gobject
-import subprocess
+import webbrowser
 
-from bzrlib.plugins.gtk import icon_path
 from bzrlib.osutils import format_date
 from bzrlib.util.bencode import bdecode
+from bzrlib.testament import Testament
+
+from bzrlib.plugins.gtk import icon_path
 
 try:
     from bzrlib.plugins.gtk import seahorse
@@ -38,8 +40,12 @@ PAGE_RELATIONS = 1
 PAGE_SIGNATURE = 2
 PAGE_BUGS = 3
 
+
 def _open_link(widget, uri):
-    subprocess.Popen(['sensible-browser', uri], close_fds=True)
+    for cmd in ['sensible-browser', 'xdg-open']:
+        if webbrowser._iscommand(cmd):
+            webbrowser._tryorder.insert(0, '%s "%%s"' % cmd)
+    webbrowser.open(uri)
 
 gtk.link_button_set_uri_hook(_open_link)
 
@@ -216,7 +222,17 @@ class SignatureTab(gtk.VBox):
                                         "This revision has not been signed.")
 
     def show_signature(self, crypttext):
-        key = seahorse.verify(crypttext)
+        (cleartext, key) = seahorse.verify(crypttext)
+
+        assert cleartext is not None
+
+        inv = self.repository.get_inventory(self.revision.revision_id)
+        expected_testament = Testament(self.revision, inv).as_short_text()
+        if expected_testament != cleartext:
+            self.signature_image.set_from_file(icon_path("sign-bad.png"))
+            self.signature_label.set_markup("<b>Signature does not match repository data</b>\n" +
+                        "The signature plaintext is different from the expected testament plaintext.")
+            return
 
         if key and key.is_available():
             if key.is_trusted():
@@ -318,8 +334,8 @@ class RevisionView(gtk.Notebook):
         self._create_general()
         self._create_relations()
         # Disabled because testaments aren't verified yet:
-        # if has_seahorse:
-        #    self._create_signature()
+        if has_seahorse:
+            self._create_signature()
         self._create_file_info_view()
         self._create_bugs()
 
@@ -493,7 +509,7 @@ class RevisionView(gtk.Notebook):
         table.resize(max(len(revids), 1), 2)
 
         for idx, revid in enumerate(revids):
-            align = gtk.Alignment(0.0, 0.0)
+            align = gtk.Alignment(0.0, 0.0, 1, 1)
             widgets.append(align)
             table.attach(align, 1, 2, idx, idx + 1,
                                       gtk.EXPAND | gtk.FILL, gtk.FILL)
@@ -516,12 +532,16 @@ class RevisionView(gtk.Notebook):
                 hbox.pack_start(button, expand=False, fill=True)
                 button.show()
 
-            button = gtk.Button(revid)
+            button = gtk.Button()
+            revid_label = gtk.Label(str(revid))
+            revid_label.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
+            revid_label.set_alignment(0.0, 0.5)
+            button.add(revid_label)
             button.connect("clicked",
                     lambda w, r: self.set_revision(self._repository.get_revision(r)), revid)
             button.set_use_underline(False)
-            hbox.pack_start(button, expand=False, fill=True)
-            button.show()
+            hbox.pack_start(button, expand=True, fill=True)
+            button.show_all()
 
     def _create_general(self):
         vbox = gtk.VBox(False, 6)
@@ -550,102 +570,91 @@ class RevisionView(gtk.Notebook):
         self.table.set_col_spacings(6)
         self.table.show()
 
-        align = gtk.Alignment(1.0, 0.5)
+        row = 0
+
         label = gtk.Label()
+        label.set_alignment(1.0, 0.5)
         label.set_markup("<b>Revision Id:</b>")
-        align.add(label)
-        self.table.attach(align, 0, 1, 0, 1, gtk.FILL, gtk.FILL)
-        align.show()
+        self.table.attach(label, 0, 1, row, row+1, gtk.FILL, gtk.FILL)
         label.show()
 
-        align = gtk.Alignment(0.0, 0.5)
         revision_id = gtk.Label()
+        revision_id.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
+        revision_id.set_alignment(0.0, 0.5)
         revision_id.set_selectable(True)
         self.connect('notify::revision', 
                 lambda w, p: revision_id.set_text(self._revision.revision_id))
-        align.add(revision_id)
-        self.table.attach(align, 1, 2, 0, 1, gtk.EXPAND | gtk.FILL, gtk.FILL)
-        align.show()
+        self.table.attach(revision_id, 1, 2, row, row+1, gtk.EXPAND | gtk.FILL, gtk.FILL)
         revision_id.show()
 
-        align = gtk.Alignment(1.0, 0.5)
+        row += 1
         self.author_label = gtk.Label()
+        self.author_label.set_alignment(1.0, 0.5)
         self.author_label.set_markup("<b>Author:</b>")
-        align.add(self.author_label)
-        self.table.attach(align, 0, 1, 1, 2, gtk.FILL, gtk.FILL)
-        align.show()
+        self.table.attach(self.author_label, 0, 1, row, row+1, gtk.FILL, gtk.FILL)
         self.author_label.show()
 
-        align = gtk.Alignment(0.0, 0.5)
         self.author = gtk.Label()
+        self.author.set_ellipsize(pango.ELLIPSIZE_END)
+        self.author.set_alignment(0.0, 0.5)
         self.author.set_selectable(True)
-        align.add(self.author)
-        self.table.attach(align, 1, 2, 1, 2, gtk.EXPAND | gtk.FILL, gtk.FILL)
-        align.show()
+        self.table.attach(self.author, 1, 2, row, row+1, gtk.EXPAND | gtk.FILL, gtk.FILL)
         self.author.show()
         self.author.hide()
 
-        align = gtk.Alignment(1.0, 0.5)
+        row += 1
         label = gtk.Label()
+        label.set_alignment(1.0, 0.5)
         label.set_markup("<b>Committer:</b>")
-        align.add(label)
-        self.table.attach(align, 0, 1, 2, 3, gtk.FILL, gtk.FILL)
-        align.show()
+        self.table.attach(label, 0, 1, row, row+1, gtk.FILL, gtk.FILL)
         label.show()
 
-        align = gtk.Alignment(0.0, 0.5)
         self.committer = gtk.Label()
+        self.committer.set_ellipsize(pango.ELLIPSIZE_END)
+        self.committer.set_alignment(0.0, 0.5)
         self.committer.set_selectable(True)
-        align.add(self.committer)
-        self.table.attach(align, 1, 2, 2, 3, gtk.EXPAND | gtk.FILL, gtk.FILL)
-        align.show()
+        self.table.attach(self.committer, 1, 2, row, row+1, gtk.EXPAND | gtk.FILL, gtk.FILL)
         self.committer.show()
 
-        align = gtk.Alignment(0.0, 0.5)
+        row += 1
         label = gtk.Label()
+        label.set_alignment(1.0, 0.5)
         label.set_markup("<b>Branch nick:</b>")
-        align.add(label)
-        self.table.attach(align, 0, 1, 3, 4, gtk.FILL, gtk.FILL)
+        self.table.attach(label, 0, 1, row, row+1, gtk.FILL, gtk.FILL)
         label.show()
-        align.show()
 
-        align = gtk.Alignment(0.0, 0.5)
         self.branchnick_label = gtk.Label()
+        self.branchnick_label.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
+        self.branchnick_label.set_alignment(0.0, 0.5)
         self.branchnick_label.set_selectable(True)
-        align.add(self.branchnick_label)
-        self.table.attach(align, 1, 2, 3, 4, gtk.EXPAND | gtk.FILL, gtk.FILL)
+        self.table.attach(self.branchnick_label, 1, 2, row, row+1, gtk.EXPAND | gtk.FILL, gtk.FILL)
         self.branchnick_label.show()
-        align.show()
 
-        align = gtk.Alignment(1.0, 0.5)
+        row += 1
         label = gtk.Label()
+        label.set_alignment(1.0, 0.5)
         label.set_markup("<b>Timestamp:</b>")
-        align.add(label)
-        self.table.attach(align, 0, 1, 4, 5, gtk.FILL, gtk.FILL)
-        align.show()
+        self.table.attach(label, 0, 1, row, row+1, gtk.FILL, gtk.FILL)
         label.show()
 
-        align = gtk.Alignment(0.0, 0.5)
         self.timestamp = gtk.Label()
+        self.timestamp.set_ellipsize(pango.ELLIPSIZE_END)
+        self.timestamp.set_alignment(0.0, 0.5)
         self.timestamp.set_selectable(True)
-        align.add(self.timestamp)
-        self.table.attach(align, 1, 2, 4, 5, gtk.EXPAND | gtk.FILL, gtk.FILL)
-        align.show()
+        self.table.attach(self.timestamp, 1, 2, row, row+1, gtk.EXPAND | gtk.FILL, gtk.FILL)
         self.timestamp.show()
 
-        align = gtk.Alignment(1.0, 0.5)
+        row += 1
         self.tags_label = gtk.Label()
+        self.tags_label.set_alignment(1.0, 0.5)
         self.tags_label.set_markup("<b>Tags:</b>")
-        align.add(self.tags_label)
-        align.show()
-        self.table.attach(align, 0, 1, 5, 6, gtk.FILL, gtk.FILL)
+        self.table.attach(self.tags_label, 0, 1, row, row+1, gtk.FILL, gtk.FILL)
         self.tags_label.show()
 
-        align = gtk.Alignment(0.0, 0.5)
         self.tags_list = gtk.Label()
-        align.add(self.tags_list)
-        self.table.attach(align, 1, 2, 5, 6, gtk.EXPAND | gtk.FILL, gtk.FILL)
-        align.show()
+        self.tags_list.set_ellipsize(pango.ELLIPSIZE_MIDDLE)
+        self.tags_list.set_alignment(0.0, 0.5)
+        self.table.attach(self.tags_list, 1, 2, row, row+1, gtk.EXPAND | gtk.FILL, gtk.FILL)
         self.tags_list.show()
 
         self.connect('notify::revision', self._add_tags)
