@@ -27,13 +27,11 @@ def test_suite():
     testmod_names = [
         'test_commit',
         'test_diff',
+        'test_linegraph',
         'test_preferences',
         'test_history',
         ]
 
-    if os.name == 'nt':
-        testmod_names.append("test_tortoise_bzr")
-
     result.addTest(loader.loadTestsFromModuleNames(["%s.%s" % (__name__, i) for i in testmod_names]))
     return result
 
