@@ -81,8 +81,8 @@ class TestPendingRevisions(tests.TestCaseWithTransport):
         rev_id1 = tree.commit('one')
         tree2 = tree.bzrdir.sprout('tree2').open_workingtree()
         rev_id2 = tree2.commit('two')
-        rev_id3 = tree2.commit('three')
         tree3 = tree2.bzrdir.sprout('tree3').open_workingtree()
+        rev_id3 = tree2.commit('three')
         rev_id4 = tree3.commit('four')
         rev_id5 = tree3.commit('five')
         tree.merge_from_branch(tree2.branch)
@@ -985,8 +985,7 @@ class TestCommitDialog_Commit(tests.TestCaseWithTransport):
                          ], bencode.bdecode(file_info))
 
     def test_commit_unicode_messages(self):
-        from bzrlib.tests.test_diff import UnicodeFilename
-        self.requireFeature(UnicodeFilename)
+        self.requireFeature(tests.UnicodeFilenameFeature)
 
         tree = self.make_branch_and_tree('tree')
         tree.branch.get_config().set_user_option('per_file_commits', 'true')
