@@ -13,14 +13,16 @@ import gtk
 import gobject
 import pango
 
-from bzrlib.plugins.gtk.window import Window
 from bzrlib.plugins.gtk import icon_path
+from bzrlib.plugins.gtk.branchview import TreeView, treemodel
 from bzrlib.plugins.gtk.tags import AddTagDialog
 from bzrlib.plugins.gtk.preferences import PreferencesWindow
-from bzrlib.plugins.gtk.branchview import TreeView, treemodel
+from bzrlib.plugins.gtk.revisionmenu import RevisionMenu
+from bzrlib.plugins.gtk.window import Window
+
+from bzrlib.config import BranchConfig, GlobalConfig
 from bzrlib.revision import Revision, NULL_REVISION
-from bzrlib.config import BranchConfig
-from bzrlib.config import GlobalConfig
+from bzrlib.trace import mutter
 
 class BranchWindow(Window):
     """Branch window.
@@ -46,6 +48,8 @@ class BranchWindow(Window):
         self.maxnum      = maxnum
         self.config      = GlobalConfig()
 
+        self._sizes      = {} # window and widget sizes
+
         if self.config.get_user_option('viz-compact-view') == 'yes':
             self.compact_view = True
         else:
@@ -53,12 +57,19 @@ class BranchWindow(Window):
 
         self.set_title(branch.nick + " - revision history")
 
-        # Use three-quarters of the screen by default
-        screen = self.get_screen()
-        monitor = screen.get_monitor_geometry(0)
-        width = int(monitor.width * 0.75)
-        height = int(monitor.height * 0.75)
+        # user-configured window size
+        size = self._load_size('viz-window-size')
+        if size:
+            width, height = size
+        else:
+            # Use three-quarters of the screen by default
+            screen = self.get_screen()
+            monitor = screen.get_monitor_geometry(0)
+            width = int(monitor.width * 0.75)
+            height = int(monitor.height * 0.75)
         self.set_default_size(width, height)
+        self.set_size_request(width/3, height/3)
+        self.connect("size-allocate", self._on_size_allocate, 'viz-window-size')
 
         # FIXME AndyFitz!
         icon = self.render_icon(gtk.STOCK_INDEX, gtk.ICON_SIZE_BUTTON)
@@ -66,6 +77,7 @@ class BranchWindow(Window):
 
         gtk.accel_map_add_entry("<viz>/Go/Next Revision", gtk.keysyms.Up, gtk.gdk.MOD1_MASK)
         gtk.accel_map_add_entry("<viz>/Go/Previous Revision", gtk.keysyms.Down, gtk.gdk.MOD1_MASK)
+        gtk.accel_map_add_entry("<viz>/View/Refresh", gtk.keysyms.F5, 0)
 
         self.accel_group = gtk.AccelGroup()
         self.add_accel_group(self.accel_group)
@@ -84,6 +96,12 @@ class BranchWindow(Window):
         self.next_rev_action.connect("activate", self._fwd_clicked_cb)
         self.next_rev_action.connect_accelerator()
 
+        self.refresh_action = gtk.Action("refresh", "_Refresh", "Refresh view", gtk.STOCK_REFRESH)
+        self.refresh_action.set_accel_path("<viz>/View/Refresh")
+        self.refresh_action.set_accel_group(self.accel_group)
+        self.refresh_action.connect("activate", self._refresh_clicked)
+        self.refresh_action.connect_accelerator()
+
         self.construct()
 
     def set_revision(self, revid):
@@ -95,18 +113,20 @@ class BranchWindow(Window):
         self.add(vbox)
 
         self.paned = gtk.VPaned()
-        self.paned.pack1(self.construct_top(), resize=True, shrink=False)
-        self.paned.pack2(self.construct_bottom(), resize=False, shrink=True)
+        self.paned.pack1(self.construct_top(), resize=False, shrink=True)
+        self.paned.pack2(self.construct_bottom(), resize=True, shrink=False)
         self.paned.show()
 
-        vbox.pack_start(self.construct_menubar(), expand=False, fill=True)
-        vbox.pack_start(self.construct_navigation(), expand=False, fill=True)
-        
+        nav = self.construct_navigation()
+        menubar = self.construct_menubar()
+        vbox.pack_start(menubar, expand=False, fill=True)
+        vbox.pack_start(nav, expand=False, fill=True)
+
         vbox.pack_start(self.paned, expand=True, fill=True)
         vbox.set_focus_child(self.paned)
 
         vbox.show()
-
+    
     def construct_menubar(self):
         menubar = gtk.MenuBar()
 
@@ -128,15 +148,12 @@ class BranchWindow(Window):
         edit_menuitem = gtk.MenuItem("_Edit")
         edit_menuitem.set_submenu(edit_menu)
 
-        edit_menu_find = gtk.ImageMenuItem(gtk.STOCK_FIND)
-
         edit_menu_branchopts = gtk.MenuItem("Branch Settings")
         edit_menu_branchopts.connect('activate', lambda x: PreferencesWindow(self.branch.get_config()).show())
 
         edit_menu_globopts = gtk.MenuItem("Global Settings")
         edit_menu_globopts.connect('activate', lambda x: PreferencesWindow().show())
 
-        edit_menu.add(edit_menu_find)
         edit_menu.add(edit_menu_branchopts)
         edit_menu.add(edit_menu_globopts)
 
@@ -144,17 +161,48 @@ class BranchWindow(Window):
         view_menuitem = gtk.MenuItem("_View")
         view_menuitem.set_submenu(view_menu)
 
+        view_menu_refresh = self.refresh_action.create_menu_item()
+        view_menu_refresh.connect('activate', self._refresh_clicked)
+
+        view_menu.add(view_menu_refresh)
+        view_menu.add(gtk.SeparatorMenuItem())
+
         view_menu_toolbar = gtk.CheckMenuItem("Show Toolbar")
         view_menu_toolbar.set_active(True)
+        if self.config.get_user_option('viz-toolbar-visible') == 'False':
+            view_menu_toolbar.set_active(False)
+            self.toolbar.hide()
         view_menu_toolbar.connect('toggled', self._toolbar_visibility_changed)
 
         view_menu_compact = gtk.CheckMenuItem("Show Compact Graph")
         view_menu_compact.set_active(self.compact_view)
         view_menu_compact.connect('activate', self._brokenlines_toggled_cb)
-
+        
+        view_menu_diffs = gtk.CheckMenuItem("Show Diffs")
+        view_menu_diffs.set_active(False)
+        if self.config.get_user_option('viz-show-diffs') == 'True':
+            view_menu_diffs.set_active(True)
+        view_menu_diffs.connect('toggled', self._diff_visibility_changed)
+        
+        view_menu_wide_diffs = gtk.CheckMenuItem("Wide Diffs")
+        view_menu_wide_diffs.set_active(False)
+        if self.config.get_user_option('viz-wide-diffs') == 'True':
+            view_menu_wide_diffs.set_active(True)
+        view_menu_wide_diffs.connect('toggled', self._diff_placement_changed)
+        
+        view_menu_wrap_diffs = gtk.CheckMenuItem("Wrap _Long Lines in Diffs")
+        view_menu_wrap_diffs.set_active(False)
+        if self.config.get_user_option('viz-wrap-diffs') == 'True':
+            view_menu_wrap_diffs.set_active(True)
+        view_menu_wrap_diffs.connect('toggled', self._diff_wrap_changed)
+                
         view_menu.add(view_menu_toolbar)
         view_menu.add(view_menu_compact)
         view_menu.add(gtk.SeparatorMenuItem())
+        view_menu.add(view_menu_diffs)
+        view_menu.add(view_menu_wide_diffs)
+        view_menu.add(view_menu_wrap_diffs)
+        view_menu.add(gtk.SeparatorMenuItem())
 
         self.mnu_show_revno_column = gtk.CheckMenuItem("Show Revision _Number Column")
         self.mnu_show_date_column = gtk.CheckMenuItem("Show _Date Column")
@@ -182,31 +230,16 @@ class BranchWindow(Window):
         tag_image.set_from_file(icon_path("tag-16.png"))
         self.go_menu_tags = gtk.ImageMenuItem("_Tags")
         self.go_menu_tags.set_image(tag_image)
-        self._update_tags()
+        self.treeview.connect('refreshed', lambda w: self._update_tags())
 
         go_menu.add(go_menu_next)
         go_menu.add(go_menu_prev)
         go_menu.add(gtk.SeparatorMenuItem())
         go_menu.add(self.go_menu_tags)
 
-        revision_menu = gtk.Menu()
+        self.revision_menu = RevisionMenu(self.branch.repository, [], self.branch, parent=self)
         revision_menuitem = gtk.MenuItem("_Revision")
-        revision_menuitem.set_submenu(revision_menu)
-
-        revision_menu_diff = gtk.MenuItem("View Changes")
-        revision_menu_diff.connect('activate', 
-                self._menu_diff_cb)
-        
-        revision_menu_compare = gtk.MenuItem("Compare with...")
-        revision_menu_compare.connect('activate',
-                self._compare_with_cb)
-
-        revision_menu_tag = gtk.MenuItem("Tag Revision")
-        revision_menu_tag.connect('activate', self._tag_revision_cb)
-
-        revision_menu.add(revision_menu_tag)
-        revision_menu.add(revision_menu_diff)
-        revision_menu.add(revision_menu_compare)
+        revision_menuitem.set_submenu(self.revision_menu)
 
         branch_menu = gtk.Menu()
         branch_menuitem = gtk.MenuItem("_Branch")
@@ -215,6 +248,21 @@ class BranchWindow(Window):
         branch_menu.add(gtk.MenuItem("Pu_ll Revisions"))
         branch_menu.add(gtk.MenuItem("Pu_sh Revisions"))
 
+        try:
+            from bzrlib.plugins import search
+        except ImportError:
+            mutter("Didn't find search plugin")
+        else:
+            branch_menu.add(gtk.SeparatorMenuItem())
+
+            branch_index_menuitem = gtk.MenuItem("_Index")
+            branch_index_menuitem.connect('activate', self._branch_index_cb)
+            branch_menu.add(branch_index_menuitem)
+
+            branch_search_menuitem = gtk.MenuItem("_Search")
+            branch_search_menuitem.connect('activate', self._branch_search_cb)
+            branch_menu.add(branch_search_menuitem)
+
         help_menu = gtk.Menu()
         help_menuitem = gtk.MenuItem("_Help")
         help_menuitem.set_submenu(help_menu)
@@ -260,6 +308,15 @@ class BranchWindow(Window):
         align = gtk.Alignment(0.0, 0.0, 1.0, 1.0)
         align.set_padding(5, 0, 0, 0)
         align.add(self.treeview)
+        # user-configured size
+        size = self._load_size('viz-graph-size')
+        if size:
+            width, height = size
+            align.set_size_request(width, height)
+        else:
+            (width, height) = self.get_size()
+            align.set_size_request(width, int(height / 2.5))
+        align.connect('size-allocate', self._on_size_allocate, 'viz-graph-size')
         align.show()
 
         return align
@@ -287,15 +344,37 @@ class BranchWindow(Window):
 
     def construct_bottom(self):
         """Construct the bottom half of the window."""
+        if self.config.get_user_option('viz-wide-diffs') == 'True':
+            self.diff_paned = gtk.VPaned()
+        else:
+            self.diff_paned = gtk.HPaned()
+        (width, height) = self.get_size()
+        self.diff_paned.set_size_request(20, 20) # shrinkable
+
         from bzrlib.plugins.gtk.revisionview import RevisionView
         self.revisionview = RevisionView(branch=self.branch)
-        (width, height) = self.get_size()
-        self.revisionview.set_size_request(width, int(height / 2.5))
+        self.revisionview.set_size_request(width/3, int(height / 2.5))
+        # user-configured size
+        size = self._load_size('viz-revisionview-size')
+        if size:
+            width, height = size
+            self.revisionview.set_size_request(width, height)
+        self.revisionview.connect('size-allocate', self._on_size_allocate, 'viz-revisionview-size')
         self.revisionview.show()
         self.revisionview.set_show_callback(self._show_clicked_cb)
         self.revisionview.connect('notify::revision', self._go_clicked_cb)
         self.treeview.connect('tag-added', lambda w, t, r: self.revisionview.update_tags())
-        return self.revisionview
+        self.diff_paned.pack1(self.revisionview)
+
+        from bzrlib.plugins.gtk.diff import DiffWidget
+        self.diff = DiffWidget()
+        self.diff_paned.pack2(self.diff)
+
+        self.diff_paned.show_all()
+        if self.config.get_user_option('viz-show-diffs') != 'True':
+            self.diff.hide()
+
+        return self.diff_paned
 
     def _tag_selected_cb(self, menuitem, revid):
         self.treeview.set_revision_id(revid)
@@ -306,6 +385,8 @@ class BranchWindow(Window):
         parents  = self.treeview.get_parents()
         children = self.treeview.get_children()
 
+        self.revision_menu.set_revision_ids([revision.revision_id])
+
         if revision and revision != NULL_REVISION:
             prev_menu = gtk.Menu()
             if len(parents) > 0:
@@ -350,7 +431,8 @@ class BranchWindow(Window):
 
             self.revisionview.set_revision(revision)
             self.revisionview.set_children(children)
-    
+            self.update_diff_panel(revision, parents)
+
     def _tree_revision_activated(self, widget, path, col):
         # TODO: more than one parent
         """Callback for when a treeview row gets activated."""
@@ -365,19 +447,6 @@ class BranchWindow(Window):
         self.show_diff(revision.revision_id, parent_id)
         self.treeview.grab_focus()
         
-    def _menu_diff_cb(self,w):
-        (path, focus) = self.treeview.treeview.get_cursor()
-        revid = self.treeview.model[path][treemodel.REVID]
-        
-        parentids = self.branch.repository.revision_parents(revid)
-
-        if len(parentids) == 0:
-            parentid = NULL_REVISION
-        else:
-            parentid = parentids[0]
-        
-        self.show_diff(revid,parentid)    
-
     def _back_clicked_cb(self, *args):
         """Callback for when the back button is clicked."""
         self.treeview.back()
@@ -396,22 +465,6 @@ class BranchWindow(Window):
         self.show_diff(revid, parentid)
         self.treeview.grab_focus()
 
-    def _compare_with_cb(self,w):
-        """Callback for revision 'compare with' menu. Will show a small
-            dialog with branch revisions to compare with selected revision in TreeView"""
-        
-        from bzrlib.plugins.gtk.revbrowser import RevisionBrowser
-        
-        rb = RevisionBrowser(self.branch,self)
-        ret = rb.run()
-        
-        if ret == gtk.RESPONSE_OK:          
-            (path, focus) = self.treeview.treeview.get_cursor()
-            revid = self.treeview.model[path][treemodel.REVID]
-            self.show_diff(revid, rb.selected_revid)
-            
-        rb.destroy()
-            
     def _set_revision_cb(self, w, revision_id):
         self.treeview.set_revision_id(revision_id)
 
@@ -427,21 +480,35 @@ class BranchWindow(Window):
         self.treeview.set_property('compact', self.compact_view)
         self.treeview.refresh()
 
-    def _tag_revision_cb(self, w):
-        try:
-            self.treeview.set_sensitive(False)
-            dialog = AddTagDialog(self.branch.repository, self.treeview.get_revision().revision_id, self.branch)
-            response = dialog.run()
-            if response != gtk.RESPONSE_NONE:
-                dialog.hide()
-            
-                if response == gtk.RESPONSE_OK:
-                    self.treeview.add_tag(dialog.tagname, dialog._revid)
-                
-                dialog.destroy()
+    def _branch_index_cb(self, w):
+        from bzrlib.plugins.search import index as _mod_index
+        _mod_index.index_url(self.branch.base)
 
-        finally:
-            self.treeview.set_sensitive(True)
+    def _branch_search_cb(self, w):
+        from bzrlib.plugins.search import index as _mod_index
+        from bzrlib.plugins.gtk.search import SearchDialog
+        from bzrlib.plugins.search import errors as search_errors
+
+        try:
+            index = _mod_index.open_index_url(self.branch.base)
+        except search_errors.NoSearchIndex:
+            dialog = gtk.MessageDialog(self, type=gtk.MESSAGE_QUESTION, 
+                buttons=gtk.BUTTONS_OK_CANCEL, 
+                message_format="This branch has not been indexed yet. "
+                               "Index now?")
+            if dialog.run() == gtk.RESPONSE_OK:
+                dialog.destroy()
+                index = _mod_index.index_url(self.branch.base)
+            else:
+                dialog.destroy()
+                return
+
+        dialog = SearchDialog(index)
+        
+        if dialog.run() == gtk.RESPONSE_OK:
+            self.set_revision(dialog.get_revision())
+
+        dialog.destroy()
 
     def _about_dialog_cb(self, w):
         from bzrlib.plugins.gtk.about import AboutDialog
@@ -454,10 +521,44 @@ class BranchWindow(Window):
 
     def _toolbar_visibility_changed(self, col):
         if col.get_active():
-            self.toolbar.show() 
+            self.toolbar.show()
         else:
             self.toolbar.hide()
+        self.config.set_user_option('viz-toolbar-visible', col.get_active())
 
+    def _make_diff_nonzero_size(self):
+        """make sure the diff isn't zero-width or zero-height"""
+        alloc = self.diff.get_allocation()
+        if (alloc.width < 10) or (alloc.height < 10):
+            width, height = self.get_size()
+            self.revisionview.set_size_request(width/3, int(height / 2.5))
+
+    def _diff_visibility_changed(self, col):
+        """Hide or show the diff panel."""
+        if col.get_active():
+            self.diff.show()
+            self._make_diff_nonzero_size()
+        else:
+            self.diff.hide()
+        self.config.set_user_option('viz-show-diffs', str(col.get_active()))
+        self.update_diff_panel()
+
+    def _diff_placement_changed(self, col):
+        """Toggle the diff panel's position."""
+        self.config.set_user_option('viz-wide-diffs', str(col.get_active()))
+
+        old = self.paned.get_child2()
+        self.paned.remove(old)
+        self.paned.pack2(self.construct_bottom(), resize=True, shrink=False)
+        self._make_diff_nonzero_size()
+
+        self.treeview.emit('revision-selected')
+    
+    def _diff_wrap_changed(self, widget):
+        """Toggle word wrap in the diff widget."""
+        self.config.set_user_option('viz-wrap-diffs', widget.get_active())
+        self.diff._on_wraplines_toggled(widget)
+    
     def _show_about_cb(self, w):
         dialog = AboutDialog()
         dialog.connect('response', lambda d,r: d.destroy())
@@ -471,14 +572,14 @@ class BranchWindow(Window):
 
         if self.branch.supports_tags():
             tags = self.branch.tags.get_tag_dict().items()
-            tags.sort()
-            tags.reverse()
+            tags.sort(reverse=True)
             for tag, revid in tags:
                 tag_image = gtk.Image()
                 tag_image.set_from_file(icon_path('tag-16.png'))
                 tag_item = gtk.ImageMenuItem(tag.replace('_', '__'))
                 tag_item.set_image(tag_image)
                 tag_item.connect('activate', self._tag_selected_cb, revid)
+                tag_item.set_sensitive(self.treeview.has_revision_id(revid))
                 menu.add(tag_item)
             self.go_menu_tags.set_submenu(menu)
 
@@ -488,6 +589,34 @@ class BranchWindow(Window):
 
         self.go_menu_tags.show_all()
 
+    def _load_size(self, name):
+        """Read and parse 'name' from self.config.
+        The value is a string, formatted as WIDTHxHEIGHT
+        Returns None, or (width, height)
+        """
+        size = self.config.get_user_option(name)
+        if size:
+            width, height = [int(num) for num in size.split('x')]
+            # avoid writing config every time we start
+            self._sizes[name] = (width, height)
+            return width, height
+        return None
+
+    def _on_size_allocate(self, widget, allocation, name):
+        """When window has been resized, save the new size."""
+        width, height = 0, 0
+        if name in self._sizes:
+            width, height = self._sizes[name]
+
+        size_changed = (width != allocation.width) or \
+                (height != allocation.height)
+
+        if size_changed:
+            width, height = allocation.width, allocation.height
+            self._sizes[name] = (width, height)
+            value = '%sx%s' % (width, height)
+            self.config.set_user_option(name, value)
+
     def show_diff(self, revid=None, parentid=None):
         """Open a new window to show a diff between the given revisions."""
         from bzrlib.plugins.gtk.diff import DiffWindow
@@ -503,4 +632,27 @@ class BranchWindow(Window):
         window.set_diff(description, rev_tree, parent_tree)
         window.show()
 
+    def update_diff_panel(self, revision=None, parents=None):
+        """Show the current revision in the diff panel."""
+        if self.config.get_user_option('viz-show-diffs') != 'True':
+            return
 
+        if not revision: # default to selected row
+            revision = self.treeview.get_revision()
+        if (not revision) or (revision == NULL_REVISION):
+            return
+
+        if not parents: # default to selected row's parents
+            parents  = self.treeview.get_parents()
+        if len(parents) == 0:
+            parent_id = None
+        else:
+            parent_id = parents[0]
+
+        rev_tree    = self.branch.repository.revision_tree(revision.revision_id)
+        parent_tree = self.branch.repository.revision_tree(parent_id)
+
+        self.diff.set_diff(rev_tree, parent_tree)
+        if self.config.get_user_option('viz-wrap-diffs') == 'True':
+            self.diff._on_wraplines_toggled(wrap=True)
+        self.diff.show_all()
